#include <stdio.h>
#include <windows.h>
#include <vector>
#include <string>
#include "Monitors.h"

#define for if(0) ; else for

static const char *p_s_class_name = "PaddingWndClass";
static const int MONITOR_ALL = 1000;

static const int MONITOR_ON = -1;
static const int MONITOR_OFF = 2;

bool Register_WndClass()
{
	WNDCLASSEX t_wnd_class;
	t_wnd_class.cbSize = sizeof(WNDCLASSEX);
	t_wnd_class.style = CS_HREDRAW | CS_VREDRAW;
	t_wnd_class.lpfnWndProc = DefWindowProc;
	t_wnd_class.cbClsExtra = 0;
	t_wnd_class.cbWndExtra = 0;
	t_wnd_class.hInstance = GetModuleHandle(NULL);
	t_wnd_class.hIcon = LoadIcon(0, IDI_APPLICATION);
	t_wnd_class.hCursor = LoadCursor(0, IDC_ARROW);
	t_wnd_class.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	t_wnd_class.lpszMenuName = 0;
	t_wnd_class.lpszClassName = p_s_class_name;
	t_wnd_class.hIconSm = LoadIcon(0, IDI_APPLICATION);
	if(!RegisterClassEx(&t_wnd_class))
		return false;
	// register window class

	return true;
}

HWND h_MakeWindow(int n_x, int n_y, int n_width, int n_heigth,
	const char *p_s_window_name = "", bool b_window_frame = false, bool b_topmost = true)
{
	HWND h_wnd = CreateWindowEx((b_topmost)? WS_EX_TOPMOST : 0, p_s_class_name,
		p_s_window_name, (b_window_frame)? WS_OVERLAPPEDWINDOW : WS_POPUP,
		n_x, n_y, n_width, n_heigth, NULL, NULL, GetModuleHandle(NULL), NULL);
	if(!h_wnd)
		return NULL;
	ShowWindow(h_wnd, SW_SHOW);
	UpdateWindow(h_wnd);
	return h_wnd;
}

void DoEvents()
{
	MSG t_msg;
	while(PeekMessage(&t_msg, NULL, 0, 0, PM_REMOVE)) {
		TranslateMessage(&t_msg);
		DispatchMessage(&t_msg);
	}
}

void TrimSpace(std::string &r_s_string)
{
	int b = 0, e = r_s_string.length();
	while(e > 0 && isspace(r_s_string[e - 1]))
		-- e;
	while(b < e && isspace(r_s_string[b]))
		++ b;
	r_s_string.erase(e);
	r_s_string.erase(0, b);
}

bool ReadLine(FILE *p_fr, std::string &r_s_line, int &r_n_cur_line)
{
	while(!feof(p_fr)) {
		r_s_line.erase();
		try {
			for(int c = fgetc(p_fr); c != '\n' && c != EOF; c = fgetc(p_fr))
				r_s_line += c;
		} catch(std::bad_alloc&) {
			return false;
		}
		// read line

		++ r_n_cur_line;
		// line counter for file debugging

		if(r_s_line.find('#') != std::string::npos)
			r_s_line.erase(r_s_line.find('#'));
		// throw away line comment

		TrimSpace(r_s_line);
		// throw away begin / end whitespace

		if(!r_s_line.length())
			continue;
		// skip empty lines

		return true;
	}

	return false;
}

int main(int n_arg_num, const char **p_arg_list)
{
	bool b_hide_taskbar = false;
	int n_max_width = -1;
	int n_max_height = -1; // max width / height constraints
	std::vector<int> keep_running_list;
	int n_run_on_monitor = 0;
	const char *p_s_mplayer_wnd_name = "MPlayer - The Movie Player";
	const char *p_s_mon_list_file = 0;
	int n_align = 0;

	// todo - implement reading logical monitor list from text file (to bypass multimon when not needed / to gain more freedom)

	for(int i = 1; i < n_arg_num; ++ i) {
		if(!strcmp(p_arg_list[i], "--hide-taskbar"))
			b_hide_taskbar = true;
		else if(!strcmp(p_arg_list[i], "--help")) {
			printf("use: SingleScreen_Mplayer [--help] [--hide-taskbar] [--mplayer-wnd-name <name>]\n"
				"\t[--max-width <nn>] [--max-height <nn>] [--keep-running <nn>] [--run-on all|<nn>]\n"
				"\t[--monitor-list <monitor-list-file>] [--align left|top|center|right|bottom]\n\n"
				"\twhere <name> is string and <nn> is integer number and <monitor-list-file> is\n"
				"\tCSV file with the following format:\n\n"
				"\tmon-name;x;y;width;height\n\n"
				"\texample file may look like:\n\n"
				"\tViewSonic VP930;0;0;1280;1024\n"
				"\tBenQ FP93G X;1280;0;1280;1024\n"
				"\tthere is one line per monitor, there must be at least one line. first monitor\n"
				"\tspecified is \"primary\". note mon-name field must not contain semicolons.\n");
			return 0;
		} else if(!strcmp(p_arg_list[i], "--mplayer-wnd-name")) {
			if(i + 1 == n_arg_num) {
				fprintf(stderr, "error: argument %s needs value\n", p_arg_list[i]);
				return -1;
			}
			p_s_mplayer_wnd_name = p_arg_list[++ i];
		} else if(!strcmp(p_arg_list[i], "--monitor-list")) {
			if(i + 1 == n_arg_num) {
				fprintf(stderr, "error: argument %s needs value\n", p_arg_list[i]);
				return -1;
			}
			p_s_mon_list_file = p_arg_list[++ i];
		} else if(!strcmp(p_arg_list[i], "--max-width")) {
			if(i + 1 == n_arg_num) {
				fprintf(stderr, "error: argument %s needs value\n", p_arg_list[i]);
				return -1;
			}
			n_max_width = atol(p_arg_list[++ i]);
		} else if(!strcmp(p_arg_list[i], "--max-height")) {
			if(i + 1 == n_arg_num) {
				fprintf(stderr, "error: argument %s needs value\n", p_arg_list[i]);
				return -1;
			}
			n_max_height = atol(p_arg_list[++ i]);
		} else if(!strcmp(p_arg_list[i], "--keep-running")) {
			if(i + 1 == n_arg_num) {
				fprintf(stderr, "error: argument %s needs value\n", p_arg_list[i]);
				return -1;
			}
			keep_running_list.push_back(atol(p_arg_list[++ i]));
		} else if(!strcmp(p_arg_list[i], "--run-on")) {
			if(i + 1 == n_arg_num) {
				fprintf(stderr, "error: argument %s needs value\n", p_arg_list[i]);
				return -1;
			}
			if(!strcmp(p_arg_list[++ i], "all"))
				n_run_on_monitor = MONITOR_ALL;
			else {
				n_run_on_monitor = atol(p_arg_list[i]);
			}
		} else if(!strcmp(p_arg_list[i], "--align")) {
			if(i + 1 == n_arg_num) {
				fprintf(stderr, "error: argument %s needs value\n", p_arg_list[i]);
				return -1;
			}
			++ i;
			if(!strcmpi(p_arg_list[i], "top") || !strcmpi(p_arg_list[i], "left"))
				n_align = -1;
			else if(!strcmpi(p_arg_list[i], "center"))
				n_align = 0;
			else if(!strcmpi(p_arg_list[i], "bottom") || !strcmpi(p_arg_list[i], "right"))
				n_align = 1;
			else {
				fprintf(stderr, "error: bad align: %s\n", p_arg_list[i]);
				return -1;
			}
		} else {
			fprintf(stderr, "error: argument %s: unknown argument\n", p_arg_list[i]);
			return -1;
		}
	}
	// parse cmdline

	std::vector<CMultimonInfo::TMonitor> monitor_list;
	CMultimonInfo::TMonitor t_virtual_screen;
	if(p_s_mon_list_file) {
		FILE *p_fr;
		if(!(p_fr = fopen(p_s_mon_list_file, "r"))) {
			fprintf(stderr, "error: failed to open \'%s\'\n", p_s_mon_list_file);
			return -1;
		}
		int n_cur_line = 0;
		std::string s_line;
		while(ReadLine(p_fr, s_line, n_cur_line)) {
			CMultimonInfo::TMonitor t_mon = {0};
			t_mon.n_bit_depth = 32;
			t_mon.n_refresh_rate = 60;
			t_mon.b_primary = monitor_list.empty();

			int n_name_length;
			if((n_name_length = s_line.find(';')) == std::string::npos) {
				fclose(p_fr);
				fprintf(stderr, "error: error reading \'%s\' (error on line %d)\n",
					p_s_mon_list_file, n_cur_line + 1);
				return -1;
			}
			s_line[n_name_length] = 0;
			strncpy(t_mon.p_s_name, s_line.data(), sizeof(t_mon.p_s_name) / sizeof(char));
			t_mon.p_s_name[sizeof(t_mon.p_s_name) / sizeof(char) - 1] = 0;
			s_line.erase(0, n_name_length + 1);

			if(sscanf(s_line.c_str(), "%d;%d;%d;%d", &t_mon.n_x,
			   &t_mon.n_y, &t_mon.n_width, &t_mon.n_height) != 4) {
				fclose(p_fr);
				fprintf(stderr, "error: error reading \'%s\' (error on line %d)\n",
					p_s_mon_list_file, n_cur_line + 1);
				return -1;
			}
			monitor_list.push_back(t_mon);
		}
		if(ferror(p_fr)) {
			fclose(p_fr);
			fprintf(stderr, "error: error reading \'%s\'\n", p_s_mon_list_file);
			return -1;
		}
		fclose(p_fr);
		// read monitor list

		if(monitor_list.empty()) {
			fprintf(stderr, "error: file \'%s\' contains no monitors\n", p_s_mon_list_file);
			return -1;
		}
		// there must be at least a single monitor

		t_virtual_screen = monitor_list[0];
		t_virtual_screen.n_width += t_virtual_screen.n_x;
		t_virtual_screen.n_height += t_virtual_screen.n_y;
		for(int i = 0, n = monitor_list.size(); i < n; ++ i) {
			if(t_virtual_screen.n_x > monitor_list[i].n_x)
				t_virtual_screen.n_x = monitor_list[i].n_x;
			if(t_virtual_screen.n_y > monitor_list[i].n_y)
				t_virtual_screen.n_y = monitor_list[i].n_y;
			int n_w = monitor_list[i].n_x + monitor_list[i].n_width;
			int n_h = monitor_list[i].n_y + monitor_list[i].n_height;
			if(t_virtual_screen.n_width < n_w)
				t_virtual_screen.n_width = n_w;
			if(t_virtual_screen.n_height < n_h)
				t_virtual_screen.n_height = n_h;
		}
		t_virtual_screen.n_width -= t_virtual_screen.n_x;
		t_virtual_screen.n_height -= t_virtual_screen.n_y;
		// find bounding box of all monitors
	} else {
		CMultimonInfo info;
		if(info.n_Monitor_Num() < 1) {
			fprintf(stderr, "error: Update_Monitors() failed\n");
			return -1;
		}
		// get info about monitors

		t_virtual_screen = info.t_Virtual_Screen();
		try {
			monitor_list.resize(info.n_Monitor_Num());
		} catch(std::bad_alloc&) {
			fprintf(stderr, "error: not enough memory\n");
			return -1;
		}
		for(int i = 0, n = info.n_Monitor_Num(); i < n; ++ i)
			monitor_list[i] = info.t_Monitor(i);
		// copy monitors from multimon
	}
	// load monitors

	if(n_run_on_monitor != MONITOR_ALL && (n_run_on_monitor < 0 ||
	   n_run_on_monitor >= monitor_list.size())) {
		fprintf(stderr, "error: specified invalid "
			"monitor to run on: %d\n", n_run_on_monitor);
		return -1;
	}
	// check params

	if(p_s_mon_list_file)
		printf("there are %d monitors in \'%s\'\n", monitor_list.size(), p_s_mon_list_file);
	else
		printf("there are %d monitors\n", monitor_list.size());
	{
		const CMultimonInfo::TMonitor &t_monitor = t_virtual_screen;
		printf("virtual-screen(size: %d x %d, position: (%d, %d))\n",
			t_monitor.n_width, t_monitor.n_height, t_monitor.n_x, t_monitor.n_y);
	}
	for(int i = 0, n = monitor_list.size(); i < n; ++ i) {
		const CMultimonInfo::TMonitor &t_monitor = monitor_list[i];
		printf("monitor(id: %d, name: \'%s\', size: %d x %d, position: (%d, %d),"
			" colors: %d bit, refresh: %d Hz)\n", i, t_monitor.p_s_name,
			t_monitor.n_width, t_monitor.n_height, t_monitor.n_x, t_monitor.n_y,
			t_monitor.n_bit_depth, t_monitor.n_refresh_rate);
	}
	// dump info about monitors

	int n_monitor_off_x, n_monitor_off_y, n_monitor_width, n_monitor_height;
	if(n_run_on_monitor == MONITOR_ALL) {
		n_monitor_off_x = t_virtual_screen.n_x;
		n_monitor_off_y = t_virtual_screen.n_y;
		n_monitor_width = t_virtual_screen.n_width;
		n_monitor_height = t_virtual_screen.n_height;
		// find bounding box of all the monitors
	} else {
		n_monitor_off_x = monitor_list[n_run_on_monitor].n_x;
		n_monitor_off_y = monitor_list[n_run_on_monitor].n_y;
		n_monitor_width = monitor_list[n_run_on_monitor].n_width;
		n_monitor_height = monitor_list[n_run_on_monitor].n_height;
		// use a single monitor
	}
	// get target bounding box

	int n_movie_area_width, n_movie_area_height;
	int n_movie_width, n_movie_height;
	int n_mplayer_x, n_mplayer_y, n_mplayer_width, n_mplayer_height;
	int n_mplayer_border_width, n_mplayer_titlebar_height;
	bool b_mplayer_topmost;
	HWND h_mplayer_window;
	{
		if(n_max_width == -1)
			n_max_width = n_monitor_width;
		if(n_max_height == -1)
			n_max_height = n_monitor_height;
		// maximal width / height of mplayer window

		n_movie_area_width = std::min(n_monitor_width, n_max_width);
		n_movie_area_height = std::min(n_monitor_height, n_max_height);
		// get MPlayer area size

		if(!(h_mplayer_window = FindWindow(NULL, p_s_mplayer_wnd_name))) {
			fprintf(stderr, "error: MPlayer is not running\n");
			return -1;
		}
		// get MPlayer window

		RECT t_window, t_client;
		GetWindowRect(h_mplayer_window, &t_window);
		GetClientRect(h_mplayer_window, &t_client);
		// get rectangles

		n_movie_width = t_client.right - t_client.left;
		n_movie_height = t_client.bottom - t_client.top;
		float f_ratio = float(n_movie_width) / n_movie_height;
		// get client area size (size of the movie)

		printf("movie(size: %d x %d, aspect: %.2f)\n", n_movie_width, n_movie_height, f_ratio);
		// verbose

		n_mplayer_border_width = ((t_window.right - t_window.left) - n_movie_width) / 2;
		n_mplayer_titlebar_height = (t_window.bottom - t_window.top) -
			n_movie_height - n_mplayer_border_width;
		// get width of MPlayer border and titlebar

		if(n_movie_area_height * f_ratio <= n_movie_area_width) {
			n_movie_width = int(n_movie_area_height * f_ratio);
			n_movie_height = n_movie_area_height;
		} else {
			n_movie_width = n_movie_area_width;
			n_movie_height = int(n_movie_area_width / f_ratio);
		}
		// get new size of MPlayer window to fit first monitor

		if(n_align == -1) {
			n_mplayer_x = n_monitor_off_x;
			n_mplayer_y = n_monitor_off_y;
		} else if(n_align == 0) {
			n_mplayer_x = n_monitor_off_x + (n_movie_area_width - n_movie_width) / 2;
			n_mplayer_y = n_monitor_off_y + (n_movie_area_height - n_movie_height) / 2;
		} else if(n_align == 1) {
			n_mplayer_x = n_monitor_off_x + (n_movie_area_width - n_movie_width);
			n_mplayer_y = n_monitor_off_y + (n_movie_area_height - n_movie_height);
		}
		n_mplayer_x -= n_mplayer_border_width;
		n_mplayer_y -= n_mplayer_titlebar_height;
		n_mplayer_width = n_movie_width + 2 * n_mplayer_border_width;
		n_mplayer_height = n_movie_height + n_mplayer_border_width + n_mplayer_titlebar_height;
		// calculate new MPlayer position and size

		b_mplayer_topmost = n_mplayer_y + n_mplayer_titlebar_height < 4;
		// make MPlayer topmost in case it crosses the "start" bar

		SetWindowPos(h_mplayer_window, HWND_TOP, n_mplayer_x, n_mplayer_y,
			n_mplayer_width, n_mplayer_height, 0);
		// move MPlayer window

		printf("MPlayer has a new position (%d, %d, %d, %d)\n",
			n_mplayer_x, n_mplayer_y, n_mplayer_width, n_mplayer_height);
		// verbose

		bool b_reaction_check = true;
		if(b_reaction_check) {
			Sleep(1000);
			// wait for MPlayer to react

			GetWindowRect(h_mplayer_window, &t_window);
			GetClientRect(h_mplayer_window, &t_client);
			// get rectangles

			n_movie_width = t_client.right - t_client.left;
			n_movie_height = t_client.bottom - t_client.top;
			float f_ratio = float(n_movie_width) / n_movie_height;
			// get client area size (size of the movie)

			printf("resized-movie(size: %d x %d, aspect: %.2f)\n", n_movie_width, n_movie_height, f_ratio);
			// verbose

			n_mplayer_width = t_window.right - t_window.left;
			n_mplayer_height = t_window.bottom - t_window.top;
			n_mplayer_border_width = (n_mplayer_width - n_movie_width) / 2;
			n_mplayer_titlebar_height = n_mplayer_height - n_movie_height - n_mplayer_border_width;
			// get MPlayer window sizes

			if(n_align == -1) {
				n_mplayer_x = n_monitor_off_x;
				n_mplayer_y = n_monitor_off_y + (n_movie_area_height - n_movie_height) / 2; // always centered on y
			} else if(n_align == 0) {
				n_mplayer_x = n_monitor_off_x + (n_movie_area_width - n_movie_width) / 2;
				n_mplayer_y = n_monitor_off_y + (n_movie_area_height - n_movie_height) / 2;
			} else if(n_align == 1) {
				n_mplayer_x = n_monitor_off_x + (n_movie_area_width - n_movie_width);
				n_mplayer_y = n_monitor_off_y + (n_movie_area_height - n_movie_height) / 2; // always centered on y
			}
			n_mplayer_x -= n_mplayer_border_width;
			n_mplayer_y -= n_mplayer_titlebar_height;
			// calculate new MPlayer position and size

			b_mplayer_topmost = n_mplayer_y + n_mplayer_titlebar_height < 4;
			// make MPlayer topmost in case it crosses the "start" bar

			SetWindowPos(h_mplayer_window, HWND_TOP, n_mplayer_x, n_mplayer_y,
				n_mplayer_width, n_mplayer_height, 0);
			// move MPlayer window

			printf("MPlayer has a new position (%d, %d, %d, %d)\n",
				n_mplayer_x, n_mplayer_y, n_mplayer_width, n_mplayer_height);
			// verbose

			Sleep(1000);
			// wait for MPlayer to react

			GetWindowRect(h_mplayer_window, &t_window);
			GetClientRect(h_mplayer_window, &t_client);
			// get rectangles

			n_movie_width = t_client.right - t_client.left;
			n_movie_height = t_client.bottom - t_client.top;
			f_ratio = float(n_movie_width) / n_movie_height;
			// get client area size (size of the movie)

			printf("resized-movie(size: %d x %d, aspect: %.2f)\n", n_movie_width, n_movie_height, f_ratio);
			// verbose

			n_mplayer_x = t_window.left;
			n_mplayer_y = t_window.top;
			n_mplayer_width = t_window.right - t_window.left;
			n_mplayer_height = t_window.bottom - t_window.top;
			n_mplayer_border_width = (n_mplayer_width - n_movie_width) / 2;
			n_mplayer_titlebar_height = n_mplayer_height - n_movie_height - n_mplayer_border_width;
			// get MPlayer window sizes

			printf("MPlayer has a new position (%d, %d, %d, %d)\n",
				n_mplayer_x, n_mplayer_y, n_mplayer_width, n_mplayer_height);
			// verbose
		}
	}
	// calculate new MPlayer size

	std::vector<HWND> padding_list;
	{
		printf("adding black padding windows ...\n");
		// verbose

		if(!Register_WndClass()) {
			fprintf(stderr, "error: Register_WndClass() failed\n");
			return -1;
		}
		// register window class so we can create windows

		if(n_movie_width == n_movie_area_width) {
			int n_bottom_border = n_mplayer_y + n_movie_height + n_mplayer_titlebar_height;
			int n_top_border = n_mplayer_y - n_monitor_off_y + n_mplayer_titlebar_height;
			int n_v_align = 0;
			if(n_top_border > 0) {
				if(n_bottom_border < n_monitor_height)
					n_v_align = 0;
				else
					n_v_align = 1;
			} else {
				if(n_bottom_border < n_monitor_height)
					n_v_align = -1;
				else
					n_v_align = 1000; // no windows actually needed
			}
			if(n_v_align == -1) {
				printf("\twindow below\n");
				padding_list.push_back(h_MakeWindow(n_monitor_off_x,
					n_mplayer_y + n_movie_height + n_mplayer_titlebar_height,
					n_movie_width, n_movie_area_height - (n_mplayer_y - n_monitor_off_y +
					n_movie_height + n_mplayer_titlebar_height)));
			} else if(n_v_align == 0) {
				printf("\twindows above / below\n");
				padding_list.push_back(h_MakeWindow(n_monitor_off_x, n_monitor_off_y,
					n_movie_width, n_mplayer_y - n_monitor_off_y + n_mplayer_titlebar_height));
				padding_list.push_back(h_MakeWindow(n_monitor_off_x,
					n_mplayer_y + n_movie_height + n_mplayer_titlebar_height,
					n_movie_width, n_movie_area_height - (n_mplayer_y - n_monitor_off_y +
					n_movie_height + n_mplayer_titlebar_height)));
			} else if(n_v_align == 1) {
				printf("\twindow above\n");
				padding_list.push_back(h_MakeWindow(n_monitor_off_x, n_monitor_off_y,
					n_movie_width, n_mplayer_y - n_monitor_off_y + n_mplayer_titlebar_height));
			}
			if(n_mplayer_x + n_mplayer_border_width > n_monitor_off_x) {
				printf("\textra window at left\n");
				padding_list.push_back(h_MakeWindow(n_monitor_off_x, n_monitor_off_y,
					n_mplayer_x + n_mplayer_border_width - n_monitor_off_x, n_monitor_height));
			}
			int n_right_border = n_mplayer_x + n_mplayer_width - n_mplayer_border_width;
			if(n_right_border - n_monitor_off_x < n_monitor_width) {
				printf("\textra window at right\n");
				padding_list.push_back(h_MakeWindow(n_right_border, n_monitor_off_y,
					n_monitor_width - n_right_border + n_monitor_off_x, n_monitor_height));
			}
		} else {
			if(n_align == -1) {
				printf("\twindow at right\n");
				padding_list.push_back(h_MakeWindow(n_mplayer_x + n_movie_width +
					n_mplayer_border_width, n_monitor_off_y, n_movie_area_width - (n_mplayer_x -
					n_monitor_off_x + n_mplayer_border_width + n_movie_width), n_movie_area_height));
			} else if(n_align == 0) {
				printf("\twindows at left / at right\n");
				padding_list.push_back(h_MakeWindow(n_monitor_off_x, n_monitor_off_y,
					n_mplayer_x + n_mplayer_border_width, n_movie_area_height));
				padding_list.push_back(h_MakeWindow(n_mplayer_x + n_movie_width +
					n_mplayer_border_width, n_monitor_off_y, n_movie_area_width - (n_mplayer_x -
					n_monitor_off_x + n_mplayer_border_width + n_movie_width), n_movie_area_height));
			} else if(n_align == 1) {
				printf("\twindow at left\n");
				padding_list.push_back(h_MakeWindow(n_monitor_off_x, n_monitor_off_y,
					n_mplayer_x + n_mplayer_border_width, n_movie_area_height));
			}
			if(n_mplayer_y + n_mplayer_titlebar_height > 0) {
				printf("\textra window above\n");
				padding_list.push_back(h_MakeWindow(n_monitor_off_x, n_monitor_off_y,
					n_monitor_width, n_mplayer_y + n_mplayer_titlebar_height));
			}
			int n_bottom_border = n_mplayer_y + n_mplayer_height - n_mplayer_border_width;
			if(n_bottom_border < n_monitor_height) {
				printf("\textra window below\n");
				padding_list.push_back(h_MakeWindow(n_monitor_off_x, n_bottom_border,
					n_monitor_width, n_monitor_height - n_bottom_border));
			}
		}
		// create windows in direct vicinity of MPlayer window

		if(n_movie_area_width < n_monitor_width) {
			printf("\twindow right to player area\n");

			padding_list.push_back(h_MakeWindow(n_monitor_off_x + n_movie_area_width,
				n_monitor_off_y, n_monitor_width - n_movie_area_width, n_movie_area_height));
		}
		if(n_movie_area_height < n_monitor_height) {
			printf("\twindow below player area\n");

			padding_list.push_back(h_MakeWindow(n_monitor_off_x, n_monitor_off_y +
				n_movie_area_height, n_movie_area_width, n_monitor_height - n_movie_area_height));
		}
		if(n_movie_area_width < n_monitor_width && n_movie_area_height < n_monitor_height) {
			printf("\twindow right & below player area\n");

			padding_list.push_back(h_MakeWindow(n_monitor_off_x + n_movie_area_width,
				n_monitor_off_y + n_movie_area_height, n_monitor_width - n_movie_area_width,
				n_monitor_height - n_movie_area_height));
		}
		// create more windows to darken the first monitor

		if(n_run_on_monitor != MONITOR_ALL) {
			for(int i = 0, n = monitor_list.size(); i < n; ++ i) {
				if(std::find(keep_running_list.begin(), keep_running_list.end(),
				   i) != keep_running_list.end())
					continue;
				// do not darken monitors user requested to keep running

				if(i == n_run_on_monitor)
					continue;
				// do not darken monitor with MPlayer on it

				const CMultimonInfo::TMonitor &t_monitor = monitor_list[i];
				printf("\twindow on %s\n", t_monitor.p_s_name);

				padding_list.push_back(h_MakeWindow(t_monitor.n_x,
					t_monitor.n_y, t_monitor.n_width, t_monitor.n_height));
			}
		}
		// create windows to darken the next monitors
	}
	// create padding windows

	// @todo - create generic clipping algorithm for rectangles

	ShowCursor(false);

	printf("done. waiting for MPlayer to finish\n");
	// verbose

	DoEvents();

	if(b_mplayer_topmost) {
		for(int i = 0, n = padding_list.size(); i < n; ++ i) {
			SetWindowPos(padding_list[i], HWND_TOPMOST, 0, 0, 0, 0,
				SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
			UpdateWindow(padding_list[i]);
			SetForegroundWindow(padding_list[i]);
			DoEvents();
		}
		// set padding windows to be above MPlayer's window
	}

	SetForegroundWindow(h_mplayer_window);
	// activate MPlayer

	HWND h_taskbar_wnd = FindWindow("Shell_traywnd", NULL);

	while(IsWindow(h_mplayer_window)) {
		if(b_hide_taskbar)
			ShowWindow(h_taskbar_wnd, SW_HIDE);
		// windows shows taskbar on occasion (icon animation? don't know yet)

		DoEvents();
		Sleep(100);
	}
	// waiting for MPlayer to finish

	if(b_hide_taskbar)
		ShowWindow(h_taskbar_wnd, SW_SHOW);

	printf("finished ...\n");

	ShowCursor(true);

	for(int i = 0, n = padding_list.size(); i < n; ++ i) {
		SendMessage(padding_list[i], WM_SYSCOMMAND, SC_MONITORPOWER, MONITOR_ON);
		DestroyWindow(padding_list[i]);
	}
	// close all the windows

	DoEvents();

	return 0;
}
