/*
								+---------------------------------+
								|                                 |
								| ***  Multi-mon win support  *** |
								|                                 |
								|  Copyright   -tHE SWINe- 2006  |
								|                                 |
								|          Monitors.cpp           |
								|                                 |
								+---------------------------------+
*/

#include <windows.h>
#define COMPILE_MULTIMON_STUBS
#include <multimon.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <vector>
#include "Monitors.h"

#if defined(_MSC_VER) && !defined(__MWERKS__) && !defined(for)
#define for if(0) {} else for
#endif

/*
 *								=== CMultimonInfo ===
 */

CMultimonInfo::CMultimonInfo()
{
	if(!Update_Monitors())
		m_monitor_list.clear(); // to mark error
}

bool CMultimonInfo::Update_Monitors()
{
	m_monitor_list.clear();
	// clean-up

	strcpy(m_t_full_window.p_s_name, "<virtual screen>");
	m_t_full_window.n_x = GetSystemMetrics(SM_XVIRTUALSCREEN);
	m_t_full_window.n_y = GetSystemMetrics(SM_YVIRTUALSCREEN);
	m_t_full_window.n_width = GetSystemMetrics(SM_CXVIRTUALSCREEN);
	m_t_full_window.n_height = GetSystemMetrics(SM_CYVIRTUALSCREEN);
	m_t_full_window.n_bit_depth = 0; // unknown
	m_t_full_window.n_refresh_rate = 0; // unknown
	// get info about virtual screen

	try {
		m_monitor_list.reserve(GetSystemMetrics(SM_CMONITORS));
	} catch(std::bad_alloc&) {
		return false;
	}
	// reserve space in the list

	assert(sizeof(void*) == sizeof(int)); // may not be true for certain 64bit compilers
	if(!EnumDisplayMonitors(0, 0, _MonCallback, (int)this))
		return false;
	// enum monitors

	return true;
}

int CMultimonInfo::n_PrimaryMonitor() const
{
	std::vector<TMonitor>::const_iterator p_primary_it;
	if((p_primary_it = std::find_if(m_monitor_list.begin(),
	   m_monitor_list.end(), b_IsPrimary)) != m_monitor_list.end())
		return p_primary_it - m_monitor_list.begin();
	return -1;
}

typedef BOOL (WINAPI *GetDevicePowerState_Ptr)(HANDLE, BOOL*);
typedef BOOL (WINAPI *EnumDisplayDevices_Ptr)(LPCSTR, DWORD, PDISPLAY_DEVICE, DWORD);

#ifndef DISPLAY_DEVICE_ACTIVE
#define DISPLAY_DEVICE_ACTIVE              0x1 
#define DISPLAY_DEVICE_ATTACHED            0x2 
#endif //DISPLAY_DEVICE_ACTIVE

#ifndef DM_POSITION
#define DM_POSITION 0x00000020L
#endif //DM_POSITION

bool CMultimonInfo::MonCallback(HMONITOR h_monitor, LPRECT p_rect)
{
	MONITORINFOEX t_mon_info;
	memset(&t_mon_info, 0, sizeof(MONITORINFOEX));
	t_mon_info.cbSize = sizeof(MONITORINFOEX);
	if(!GetMonitorInfo(h_monitor, &t_mon_info))
		return false;
	// get monitor info

	TMonitor t_monitor = {0};

	strncpy(t_monitor.p_s_name, (char*)t_mon_info.szDevice, 128);
	t_monitor.p_s_name[127] = 0;
	// copy monitor (device) name

	DEVMODE t_devmode;
	if(!EnumDisplaySettings(t_monitor.p_s_name,
	   ENUM_CURRENT_SETTINGS, &t_devmode))
		return true;
	// get device mode

#if 0
	bool b_is_on = true;
	HMODULE h_lib;
	if(h_lib = LoadLibrary("User32")) {
		EnumDisplayDevices_Ptr _EnumDisplayDevices;
		if(_EnumDisplayDevices = (EnumDisplayDevices_Ptr)GetProcAddress(h_lib, "EnumDisplayDevicesA")) {
			b_is_on = false;

			/*DISPLAY_DEVICE t_dd;
			t_dd.cb = sizeof(DISPLAY_DEVICE);
			for(int i = 0; _EnumDisplayDevices(NULL, i, &t_dd, 0); ++ i) {
				if(!strcmp(t_monitor.p_s_name, (const char*)t_dd.DeviceName)) {
					b_is_on = (t_dd.StateFlags & DISPLAY_DEVICE_ACTIVE) != 0;

					DEVMODE   defaultMode;
					ZeroMemory(&defaultMode, sizeof(DEVMODE));
					defaultMode.dmSize = sizeof(DEVMODE);
					int Result = EnumDisplaySettings((const char*)t_dd.DeviceName, ENUM_REGISTRY_SETTINGS, &defaultMode);

					if((t_dd.StateFlags & DISPLAY_DEVICE_ATTACHED_TO_DESKTOP) &&
					   !(t_dd.StateFlags & DISPLAY_DEVICE_PRIMARY_DEVICE)) {
						DEVMODE DevMode;
						ZeroMemory(&DevMode, sizeof(DevMode));
						DevMode.dmSize = sizeof(DevMode);
						DevMode.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT | DM_BITSPERPEL | DM_POSITION |
							DM_DISPLAYFREQUENCY | DM_DISPLAYFLAGS;

						Result = ChangeDisplaySettingsEx((const char*)t_dd.DeviceName, 
														&DevMode,
														NULL,
														CDS_UPDATEREGISTRY,
														NULL);
						Result = ChangeDisplaySettingsEx((const char*)t_dd.DeviceName,
														&DevMode,
														NULL,
														CDS_UPDATEREGISTRY,
														NULL);

						//	The code below shows how to re-attach the secondary displays to the desktop
						//	except it doesn't work because when detached, t_dd.DeviceName no longer points
						//	to the monitor, so it doesn't wake it. bummer, would've been cool to use it.

						Result = ChangeDisplaySettingsEx((const char*)t_dd.DeviceName,
												 &defaultMode,
												 NULL,
												 CDS_UPDATEREGISTRY,
												 NULL);
						Result = ChangeDisplaySettingsEx((const char*)t_dd.DeviceName,
												 &defaultMode,
												 NULL,
												 CDS_UPDATEREGISTRY,
												 NULL);
            
						
					}

					break;
				}
			}*/
			// this shows how to detach monitor from desktop, effectively turning it off
			// unfortunately, it doesn't seem to work under xp x64
		}
		FreeLibrary(h_lib);
	}
	// this finds out wheter display device is currently "on".
	// reportedly works under windows vista, doesn't work under XP
#endif //0

	t_monitor.b_primary = (t_mon_info.dwFlags & MONITORINFOF_PRIMARY) != 0;
	t_monitor.n_bit_depth = t_devmode.dmBitsPerPel;
	t_monitor.n_refresh_rate = t_devmode.dmDisplayFrequency;
	// copy some information about the display

	t_monitor.n_x = t_mon_info.rcMonitor.left;
	t_monitor.n_y = t_mon_info.rcMonitor.top;
	t_monitor.n_width = t_mon_info.rcMonitor.right -
		t_mon_info.rcMonitor.left;
	t_monitor.n_height = t_mon_info.rcMonitor.bottom -
		t_mon_info.rcMonitor.top;
	// get monitor size / position

	try {
		m_monitor_list.push_back(t_monitor);
	} catch(std::bad_alloc&) {
		return false;
	}
	// add it to the list

	return true;
}

BOOL CALLBACK CMultimonInfo::_MonCallback(HMONITOR h_monitor, HDC h_dc, LPRECT p_rect, LPARAM n_l_param)
{
	return ((CMultimonInfo*)n_l_param)->MonCallback(h_monitor, p_rect); // h_dc is NULL, no sense in passing it
	// just call member function
}

/*
 *								=== ~CMultimonInfo ===
 */
